Method: Mongo::Session#dirty!

Defined in:
lib/mongo/session.rb

#dirty!(mark = true) ⇒ Object

Sets the dirty state to the given value for the underlying server session. If there is no server session, this does nothing.

Parameters:

  • mark (true | false) (defaults to: true)

    whether to mark the server session as dirty, or not.

Since:

  • 2.5.0



140
141
142
# File 'lib/mongo/session.rb', line 140

def dirty!(mark = true)
  @server_session&.dirty!(mark)
end