Method: Mongo::Session#dirty?

Defined in:
lib/mongo/session.rb

#dirty?true | false | nil

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Returns whether the underlying server session is dirty. If no server session exists for this session, returns nil.

Returns:

  • (true | false | nil)

    whether the underlying server session is dirty. If no server session exists for this session, returns nil.

Since:

  • 2.5.0



148
149
150
# File 'lib/mongo/session.rb', line 148

def dirty?
  @server_session&.dirty?
end