Class: Mongo::Crypt::ExplicitDecryptionContext Private
- Defined in:
- lib/mongo/crypt/explicit_decryption_context.rb
Overview
This class is part of a private API. You should avoid using this class if possible, as it may be removed or be changed in the future.
A Context object initialized for explicit decryption
Instance Attribute Summary
Attributes inherited from Context
Instance Method Summary collapse
-
#bson_mode ⇒ Symbol?
private
Which BSON mode to use when creating documents from the outcome of the state machine.
-
#initialize(mongocrypt, io, doc) ⇒ ExplicitDecryptionContext
constructor
private
Create a new ExplicitDecryptionContext object.
Methods inherited from Context
Constructor Details
#initialize(mongocrypt, io, doc) ⇒ ExplicitDecryptionContext
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Create a new ExplicitDecryptionContext object
31 32 33 34 35 36 37 |
# File 'lib/mongo/crypt/explicit_decryption_context.rb', line 31 def initialize(mongocrypt, io, doc) super(mongocrypt, io) # Initialize the underlying mongocrypt_ctx_t object to perform # explicit decryption Binding.ctx_explicit_decrypt_init(self, doc) end |
Instance Method Details
#bson_mode ⇒ Symbol?
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Which BSON mode to use when creating documents from the outcome of the state machine. The returned value is based on the Mongo::Config.csfle_convert_to_ruby_types option.
44 45 46 |
# File 'lib/mongo/crypt/explicit_decryption_context.rb', line 44 def bson_mode Mongo::Config.csfle_convert_to_ruby_types ? nil : :bson end |