Class: Mongo::Crypt::RewrapManyDataKeyResult

Inherits:
Object
  • Object
show all
Defined in:
lib/mongo/crypt/rewrap_many_data_key_result.rb

Overview

Represent result of the rewrap many data key operation.

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(bulk_write_result) ⇒ RewrapManyDataKeyResult

Returns a new instance of RewrapManyDataKeyResult.

Parameters:

  • bulk_write_result (BulkWrite::Result | nil)

    The result of the bulk write operation used to update the key vault collection with rewrapped data keys.



30
31
32
# File 'lib/mongo/crypt/rewrap_many_data_key_result.rb', line 30

def initialize(bulk_write_result)
  @bulk_write_result = bulk_write_result
end

Instance Attribute Details

#bulk_write_resultBulkWrite::Result (readonly)

Returns the result of the bulk write operation used to update the key vault collection with rewrapped data keys.

Returns:

  • (BulkWrite::Result)

    the result of the bulk write operation used to update the key vault collection with rewrapped data keys.



25
26
27
# File 'lib/mongo/crypt/rewrap_many_data_key_result.rb', line 25

def bulk_write_result
  @bulk_write_result
end