Class: Mongo::Error::InvalidURI

Inherits:
Error
  • Object
show all
Defined in:
lib/mongo/error/invalid_uri.rb

Overview

Exception that is raised when trying to parse a URI that does not match the specification.

Since:

  • 2.0.0

Instance Method Summary collapse

Constructor Details

#initialize(uri, details, format = nil) ⇒ InvalidURI

Instantiate the new exception.

The URI is redacted via URI.redact before being interpolated so that any cleartext credentials in the original input do not end up in logs, error reporters, or backtraces.

Examples:

Instantiate the exception.

Mongo::Error::InvalidURI.new(uri, details, format)

Since:

  • 2.0.0



34
35
36
37
38
39
40
# File 'lib/mongo/error/invalid_uri.rb', line 34

def initialize(uri, details, format = nil)
  message = "Bad URI: #{Mongo::URI.redact(uri)}\n" +
            "#{details}\n"
  message += "MongoDB URI must be in the following format: #{format}\n" if format
  message += "Please see the following URL for more information: #{Mongo::URI::HELP}\n"
  super(message)
end