Module: Mongo::Error::Labelable
- Included in:
- Auth::Unauthorized, Mongo::Error
- Defined in:
- lib/mongo/error/labelable.rb
Overview
Although methods of this module are part of the public API, the fact that these methods are defined on this module and not on the classes which include this module is not part of the public API.
A module encapsulating functionality to manage labels added to errors.
Instance Method Summary collapse
-
#add_label(label) ⇒ Object
private
Adds the specified label to the error instance, if the label is not already in the set of labels.
-
#label?(label) ⇒ true, false
Does the error have the given label?.
-
#labels ⇒ Array
Gets the set of labels associated with the error.
Instance Method Details
#add_label(label) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Adds the specified label to the error instance, if the label is not already in the set of labels.
63 64 65 66 |
# File 'lib/mongo/error/labelable.rb', line 63 def add_label(label) @labels ||= [] @labels << label unless label?(label) end |
#label?(label) ⇒ true, false
Does the error have the given label?
37 38 39 |
# File 'lib/mongo/error/labelable.rb', line 37 def label?(label) @labels && @labels.include?(label) end |
#labels ⇒ Array
Gets the set of labels associated with the error.
49 50 51 52 53 54 55 |
# File 'lib/mongo/error/labelable.rb', line 49 def labels if @labels @labels.dup else [] end end |