Class: Mongo::Error::UnsupportedArrayFilters Deprecated

Inherits:
UnsupportedOption show all
Defined in:
lib/mongo/error/unsupported_array_filters.rb

Overview

Deprecated.

RUBY-2260 In driver version 3.0, this error class will be replaced with UnsupportedOption. To handle this error, catch Mongo::Error::UnsupportedOption, which will prevent any breaking changes in your application when upgrading to version 3.0 of the driver.

Raised if the array filters option is specified for an operation but the server selected does not support array filters.

Since:

  • 2.5.0

Constant Summary collapse

UNACKNOWLEDGED_WRITES_MESSAGE =

The error message describing that array filters cannot be used when write concern is unacknowledged.

Returns:

  • (String)

    A message describing that array filters cannot be used when write concern is unacknowledged.

Since:

  • 2.5.0

'The array_filters option cannot be specified when using unacknowledged writes. ' +
'Either remove the array_filters option or use acknowledged writes (w >= 1).'

Constants inherited from UnsupportedOption

Mongo::Error::UnsupportedOption::ALLOW_DISK_USE_MESSAGE, Mongo::Error::UnsupportedOption::COMMIT_QUORUM_MESSAGE, Mongo::Error::UnsupportedOption::HINT_MESSAGE, Mongo::Error::UnsupportedOption::UNACKNOWLEDGED_HINT_MESSAGE

Method Summary

Methods inherited from UnsupportedOption

allow_disk_use_error, commit_quorum_error, hint_error