Class: Mongo::Monitoring::Event::ServerHeartbeatSucceeded
- Inherits:
-
Event::Base
- Object
- Event::Base
- Mongo::Monitoring::Event::ServerHeartbeatSucceeded
- Defined in:
- lib/mongo/monitoring/event/server_heartbeat_succeeded.rb
Overview
Event fired when a server heartbeat is dispatched.
Instance Attribute Summary collapse
-
#address ⇒ Address
readonly
Address The server address.
-
#round_trip_time ⇒ Float
(also: #duration)
readonly
Round_trip_time Duration of hello call in seconds.
-
#started_event ⇒ Monitoring::Event::ServerHeartbeatStarted
readonly
The corresponding started event.
Instance Method Summary collapse
-
#awaited? ⇒ true | false
Whether the heartbeat was awaited.
-
#initialize(address, round_trip_time, started_event:, awaited: false) ⇒ ServerHeartbeatSucceeded
constructor
private
Create the event.
-
#summary ⇒ String
Returns a concise yet useful summary of the event.
Constructor Details
#initialize(address, round_trip_time, started_event:, awaited: false) ⇒ ServerHeartbeatSucceeded
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Create the event.
37 38 39 40 41 42 43 |
# File 'lib/mongo/monitoring/event/server_heartbeat_succeeded.rb', line 37 def initialize(address, round_trip_time, started_event:, awaited: false) super() @address = address @round_trip_time = round_trip_time @awaited = !!awaited @started_event = started_event end |
Instance Attribute Details
#address ⇒ Address (readonly)
Returns address The server address.
46 47 48 |
# File 'lib/mongo/monitoring/event/server_heartbeat_succeeded.rb', line 46 def address @address end |
#round_trip_time ⇒ Float (readonly) Also known as: duration
Returns round_trip_time Duration of hello call in seconds.
49 50 51 |
# File 'lib/mongo/monitoring/event/server_heartbeat_succeeded.rb', line 49 def round_trip_time @round_trip_time end |
#started_event ⇒ Monitoring::Event::ServerHeartbeatStarted (readonly)
Returns The corresponding started event.
63 64 65 |
# File 'lib/mongo/monitoring/event/server_heartbeat_succeeded.rb', line 63 def started_event @started_event end |
Instance Method Details
#awaited? ⇒ true | false
Returns Whether the heartbeat was awaited.
55 56 57 |
# File 'lib/mongo/monitoring/event/server_heartbeat_succeeded.rb', line 55 def awaited? @awaited end |
#summary ⇒ String
Note:
This method is experimental and subject to change.
Returns a concise yet useful summary of the event.
73 74 75 76 |
# File 'lib/mongo/monitoring/event/server_heartbeat_succeeded.rb', line 73 def summary "#<#{short_class_name}" + " address=#{address}>" end |