Module: Mongo::Operation::PolymorphicOperation Private

Included in:
CollectionsInfo, Indexes, OpMsgOrCommand, OpMsgOrFindCommand
Defined in:
lib/mongo/operation/shared/polymorphic_operation.rb

Overview

This module is part of a private API. You should avoid using this module if possible, as it may be removed or be changed in the future.

Shared behavior of implementing an operation differently based on the server that will be executing the operation.

API:

  • private

Instance Method Summary collapse

Instance Method Details

#execute(server, context:, options: {}) ⇒ Mongo::Operation::Result

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Execute the operation.

Parameters:

  • The server to send the operation to.

  • The operation context.

  • (defaults to: {})

    Operation execution options.

Returns:

  • The operation result.

API:

  • private



34
35
36
37
38
39
# File 'lib/mongo/operation/shared/polymorphic_operation.rb', line 34

def execute(server, context:, options: {})
  server.with_connection(service_id: context.service_id) do |connection|
    operation = final_operation(connection)
    operation.execute(connection, context: context, options: options)
  end
end