Module: Mongo::Operation::Result::Aggregatable Private
- Defined in:
- lib/mongo/operation/shared/result/aggregatable.rb
Overview
This module is part of a private API. You should avoid using this module if possible, as it may be removed or be changed in the future.
Defines custom behavior of bulk write results
Instance Method Summary collapse
-
#aggregate_write_concern_errors(count) ⇒ Array
private
Aggregate the write concern errors returned from this result.
-
#aggregate_write_errors(count) ⇒ Array
private
Aggregate the write errors returned from this result.
Instance Method Details
#aggregate_write_concern_errors(count) ⇒ Array
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Aggregate the write concern errors returned from this result.
58 59 60 61 62 63 64 65 66 67 68 69 |
# File 'lib/mongo/operation/shared/result/aggregatable.rb', line 58 def aggregate_write_concern_errors(count) return unless @replies @replies.each_with_index.reduce(nil) do |errors, (reply, _)| next unless write_concern_errors = reply.documents.first['writeConcernErrors'] (errors || []) << write_concern_errors.reduce(nil) do |errs, wce| wce.merge!('index' => count + wce['index']) (errs || []) << write_concern_error end end end |
#aggregate_write_errors(count) ⇒ Array
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Aggregate the write errors returned from this result.
35 36 37 38 39 40 41 42 43 44 45 46 |
# File 'lib/mongo/operation/shared/result/aggregatable.rb', line 35 def aggregate_write_errors(count) return unless @replies @replies.reduce(nil) do |errors, reply| next unless write_errors = reply.documents.first['writeErrors'] wes = write_errors.collect do |we| we.merge!('index' => count + we['index']) end (errors || []) << wes if wes end end |