Method: Mongo::Operation::Result#connection_description

Defined in:
lib/mongo/operation/result.rb

#connection_descriptionServer::Description (readonly)

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Returns Server description of the server that the operation was performed on that this result is for.

Since:

  • 2.0.0



129
130
131
# File 'lib/mongo/operation/result.rb', line 129

def connection_description
  @connection_description
end