Method: Mongo::ServerSelector::Base#max_staleness

Defined in:
lib/mongo/server_selector/base.rb

#max_stalenessInteger (readonly)

Returns max_staleness The maximum replication lag, in seconds, that a secondary can suffer and still be eligible for a read.

Returns:

  • (Integer)

    max_staleness The maximum replication lag, in seconds, that a secondary can suffer and still be eligible for a read.

Since:

  • 2.4.0



72
73
74
# File 'lib/mongo/server_selector/base.rb', line 72

def max_staleness
  @max_staleness
end