Method: Mongo::Session#dirty!
- Defined in:
- lib/mongo/session.rb
#dirty!(mark = true) ⇒ Object
Sets the dirty state to the given value for the underlying server session. If there is no server session, this does nothing.
140 141 142 |
# File 'lib/mongo/session.rb', line 140 def dirty!(mark = true) @server_session&.dirty!(mark) end |