Class: Mongo::BulkWrite::Result
- Inherits:
-
Object
- Object
- Mongo::BulkWrite::Result
- Defined in:
- lib/mongo/bulk_write/result.rb
Overview
Wraps a series of bulk write operations in a result object.
Constant Summary collapse
- REMOVED_COUNT =
Constant for number removed.
'n_removed'- INSERTED_COUNT =
Constant for number inserted.
'n_inserted'- INSERTED_IDS =
Constant for inserted ids.
'inserted_ids'- MATCHED_COUNT =
Constant for number matched.
'n_matched'- MODIFIED_COUNT =
Constant for number modified.
'n_modified'- UPSERTED =
Constant for upserted.
'upserted'- UPSERTED_COUNT =
Constant for number upserted.
'n_upserted'- UPSERTED_IDS =
Constant for upserted ids.
'upserted_ids'- FIELDS =
The fields contained in the result document returned from executing the operations.
[ INSERTED_COUNT, REMOVED_COUNT, MODIFIED_COUNT, UPSERTED_COUNT, MATCHED_COUNT, Operation::Result::N ].freeze
Instance Method Summary collapse
-
#acknowledged? ⇒ Boolean
Is the result acknowledged?.
-
#deleted_count ⇒ Integer
Returns the number of documents deleted.
-
#initialize(results, acknowledged) ⇒ Result
constructor
private
Create the new result object from the results document.
-
#inserted_count ⇒ Integer
Returns the number of documents inserted.
-
#inserted_ids ⇒ Array<BSON::ObjectId>
Get the inserted document ids, if the operation has inserts.
-
#matched_count ⇒ Integer
Returns the number of documents matched.
-
#modified_count ⇒ Integer
Returns the number of documents modified.
-
#upserted_count ⇒ Integer
Returns the number of documents upserted.
-
#upserted_ids ⇒ Array<BSON::ObjectId>
Get the upserted document ids, if the operation has inserts.
-
#validate! ⇒ Result
Validates the bulk write result.
Constructor Details
#initialize(results, acknowledged) ⇒ Result
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Create the new result object from the results document.
104 105 106 107 |
# File 'lib/mongo/bulk_write/result.rb', line 104 def initialize(results, acknowledged) @results = results @acknowledged = acknowledged end |
Instance Method Details
#acknowledged? ⇒ Boolean
Returns Is the result acknowledged?.
24 25 26 |
# File 'lib/mongo/bulk_write/result.rb', line 24 def acknowledged? @acknowledged end |
#deleted_count ⇒ Integer
Returns the number of documents deleted.
89 90 91 |
# File 'lib/mongo/bulk_write/result.rb', line 89 def deleted_count @results[REMOVED_COUNT] end |
#inserted_count ⇒ Integer
Returns the number of documents inserted.
117 118 119 |
# File 'lib/mongo/bulk_write/result.rb', line 117 def inserted_count @results[INSERTED_COUNT] end |
#inserted_ids ⇒ Array<BSON::ObjectId>
Get the inserted document ids, if the operation has inserts.
129 130 131 |
# File 'lib/mongo/bulk_write/result.rb', line 129 def inserted_ids @results[INSERTED_IDS] end |
#matched_count ⇒ Integer
Returns the number of documents matched.
141 142 143 |
# File 'lib/mongo/bulk_write/result.rb', line 141 def matched_count @results[MATCHED_COUNT] end |
#modified_count ⇒ Integer
Returns the number of documents modified.
153 154 155 |
# File 'lib/mongo/bulk_write/result.rb', line 153 def modified_count @results[MODIFIED_COUNT] end |
#upserted_count ⇒ Integer
Returns the number of documents upserted.
165 166 167 |
# File 'lib/mongo/bulk_write/result.rb', line 165 def upserted_count @results[UPSERTED_COUNT] end |
#upserted_ids ⇒ Array<BSON::ObjectId>
Get the upserted document ids, if the operation has inserts.
177 178 179 |
# File 'lib/mongo/bulk_write/result.rb', line 177 def upserted_ids @results[UPSERTED_IDS] || [] end |
#validate! ⇒ Result
Validates the bulk write result.
191 192 193 194 195 |
# File 'lib/mongo/bulk_write/result.rb', line 191 def validate! raise Error::BulkWriteError.new(@results) if @results['writeErrors'] || @results['writeConcernErrors'] self end |