Method: Mongo::Cluster#==
- Defined in:
- lib/mongo/cluster.rb
#==(other) ⇒ true, false
Determine if this cluster of servers is equal to another object. Checks the servers currently in the cluster, not what was configured.
743 744 745 746 |
# File 'lib/mongo/cluster.rb', line 743 def ==(other) return false unless other.is_a?(Cluster) addresses == other.addresses && == other. end |