Module: Mongo::Collection::View::Writable
- Included in:
- Mongo::Collection::View
- Defined in:
- lib/mongo/collection/view/writable.rb
Overview
Defines write related behaviour for collection view.
Instance Method Summary collapse
-
#delete_many(opts = {}) ⇒ Result
Remove documents from the collection.
-
#delete_one(opts = {}) ⇒ Result
Remove a document from the collection.
-
#find_one_and_delete ⇒ BSON::Document?
Finds a single document in the database via findAndModify and deletes it, returning the original document.
-
#find_one_and_replace(replacement, opts = {}) ⇒ BSON::Document
Finds a single document and replaces it.
-
#find_one_and_update(document, opts = {}) ⇒ BSON::Document
Finds a single document and updates it.
-
#replace_one(replacement, opts = {}) ⇒ Result
Replaces a single document in the database with the new document.
-
#update_many(spec, opts = {}) ⇒ Result
Update documents in the collection.
-
#update_one(spec, opts = {}) ⇒ Result
Update a single document in the collection.
Instance Method Details
#delete_many(opts = {}) ⇒ Result
Remove documents from the collection.
131 132 133 |
# File 'lib/mongo/collection/view/writable.rb', line 131 def delete_many(opts = {}) remove(0, opts) end |
#delete_one(opts = {}) ⇒ Result
Remove a document from the collection.
147 148 149 |
# File 'lib/mongo/collection/view/writable.rb', line 147 def delete_one(opts = {}) remove(1, opts) end |
#find_one_and_delete ⇒ BSON::Document?
Finds a single document in the database via findAndModify and deletes it, returning the original document.
33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 |
# File 'lib/mongo/collection/view/writable.rb', line 33 def find_one_and_delete cmd = { :findandmodify => collection.name, :query => filter, :remove => true } cmd[:fields] = projection if projection cmd[:sort] = sort if sort cmd[:maxTimeMS] = max_time_ms if max_time_ms cmd[:writeConcern] = write_concern. if write_concern server = next_primary validate_collation!(server, ) cmd[:collation] = [:collation] if [:collation] write_with_retry do Operation::Commands::Command.new({ :selector => cmd, :db_name => database.name }).execute(server).first['value'] end end |
#find_one_and_replace(replacement, opts = {}) ⇒ BSON::Document
Finds a single document and replaces it.
73 74 75 |
# File 'lib/mongo/collection/view/writable.rb', line 73 def find_one_and_replace(replacement, opts = {}) find_one_and_update(replacement, opts) end |
#find_one_and_update(document, opts = {}) ⇒ BSON::Document
Finds a single document and updates it.
95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 |
# File 'lib/mongo/collection/view/writable.rb', line 95 def find_one_and_update(document, opts = {}) cmd = { :findandmodify => collection.name, :query => filter } cmd[:update] = document cmd[:fields] = projection if projection cmd[:sort] = sort if sort cmd[:new] = !!(opts[:return_document] && opts[:return_document] == :after) cmd[:upsert] = opts[:upsert] if opts[:upsert] cmd[:maxTimeMS] = max_time_ms if max_time_ms cmd[:bypassDocumentValidation] = !!opts[:bypass_document_validation] cmd[:writeConcern] = write_concern. if write_concern server = next_primary validate_collation!(server, ) cmd[:collation] = [:collation] if [:collation] value = write_with_retry do Operation::Commands::Command.new({ :selector => cmd, :db_name => database.name }).execute(server).first['value'] end value unless value.nil? || value.empty? end |
#replace_one(replacement, opts = {}) ⇒ Result
Replaces a single document in the database with the new document.
166 167 168 |
# File 'lib/mongo/collection/view/writable.rb', line 166 def replace_one(replacement, opts = {}) update(replacement, false, opts) end |
#update_many(spec, opts = {}) ⇒ Result
Update documents in the collection.
185 186 187 |
# File 'lib/mongo/collection/view/writable.rb', line 185 def update_many(spec, opts = {}) update(spec, true, opts) end |
#update_one(spec, opts = {}) ⇒ Result
Update a single document in the collection.
204 205 206 |
# File 'lib/mongo/collection/view/writable.rb', line 204 def update_one(spec, opts = {}) update(spec, false, opts) end |