Class: Mongo::Database
- Inherits:
-
Object
- Object
- Mongo::Database
- Extended by:
- Forwardable
- Defined in:
- lib/mongo/database.rb,
lib/mongo/database/view.rb
Overview
Represents a database on the db server and operations that can execute on it at this level.
Defined Under Namespace
Classes: View
Constant Summary collapse
- ADMIN =
The admin database name.
'admin'.freeze
- COMMAND =
The “collection” that database commands operate against.
'$cmd'.freeze
- DEFAULT_OPTIONS =
The default database options.
Options::Redacted.new(:database => ADMIN).freeze
- NAME =
Database name field constant.
'name'.freeze
- DATABASES =
Databases constant.
'databases'.freeze
- NAMESPACES =
The name of the collection that holds all the collection names.
'system.namespaces'.freeze
Instance Attribute Summary collapse
-
#client ⇒ Client
readonly
Client The database client.
-
#name ⇒ String
readonly
Name The name of the database.
-
#options ⇒ Hash
readonly
Options The options.
Class Method Summary collapse
-
.create(client) ⇒ Database
private
Create a database for the provided client, for use when we don’t want the client’s original database instance to be the same.
Instance Method Summary collapse
-
#==(other) ⇒ true, false
Check equality of the database object against another.
-
#[](collection_name, options = {}) ⇒ Mongo::Collection
(also: #collection)
Get a collection in this database by the provided name.
-
#cluster ⇒ Mongo::Server
Get the primary server from the cluster.
-
#collection_names(options = {}) ⇒ Array<String>
Get all the names of the non-system collections in the database.
-
#collections ⇒ Array<Mongo::Collection>
Get all the collections that belong to this database.
-
#command(operation, opts = {}) ⇒ Hash
Execute a command on the database.
-
#drop(options = {}) ⇒ Result
Drop the database and all its associated information.
-
#fs(options = {}) ⇒ Grid::FSBucket
Get the Grid “filesystem” for this database.
-
#initialize(client, name, options = {}) ⇒ Database
constructor
Instantiate a new database object.
-
#inspect ⇒ String
Get a pretty printed string inspection for the database.
-
#list_collections ⇒ Array<Hash>
Get info on all the collections in the database.
-
#users ⇒ View::User
Get the user view for this database.
-
#watch(pipeline = [], options = {}) ⇒ ChangeStream
As of version 3.6 of the MongoDB server, a “$changeStream“ pipeline stage is supported in the aggregation framework.
Constructor Details
#initialize(client, name, options = {}) ⇒ Database
Instantiate a new database object.
207 208 209 210 211 212 |
# File 'lib/mongo/database.rb', line 207 def initialize(client, name, = {}) raise Error::InvalidDatabaseName.new unless name @client = client @name = name.to_s.freeze = .freeze end |
Instance Attribute Details
#client ⇒ Client (readonly)
Returns client The database client.
57 58 59 |
# File 'lib/mongo/database.rb', line 57 def client @client end |
#name ⇒ String (readonly)
Returns name The name of the database.
60 61 62 |
# File 'lib/mongo/database.rb', line 60 def name @name end |
#options ⇒ Hash (readonly)
Returns options The options.
63 64 65 |
# File 'lib/mongo/database.rb', line 63 def end |
Class Method Details
.create(client) ⇒ Database
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Create a database for the provided client, for use when we don’t want the client’s original database instance to be the same.
304 305 306 307 |
# File 'lib/mongo/database.rb', line 304 def self.create(client) database = Database.new(client, client.[:database], client.) client.instance_variable_set(:@database, database) end |
Instance Method Details
#==(other) ⇒ true, false
Check equality of the database object against another. Will simply check if the names are the same.
87 88 89 90 |
# File 'lib/mongo/database.rb', line 87 def ==(other) return false unless other.is_a?(Database) name == other.name end |
#[](collection_name, options = {}) ⇒ Mongo::Collection Also known as: collection
Get a collection in this database by the provided name.
103 104 105 |
# File 'lib/mongo/database.rb', line 103 def [](collection_name, = {}) Collection.new(self, collection_name, ) end |
#cluster ⇒ Mongo::Server
Returns Get the primary server from the cluster.
73 74 |
# File 'lib/mongo/database.rb', line 73 def_delegators :cluster, :next_primary |
#collection_names(options = {}) ⇒ Array<String>
Get all the names of the non-system collections in the database.
116 117 118 |
# File 'lib/mongo/database.rb', line 116 def collection_names( = {}) View.new(self).collection_names() end |
#collections ⇒ Array<Mongo::Collection>
Get all the collections that belong to this database.
140 141 142 |
# File 'lib/mongo/database.rb', line 140 def collections collection_names.map { |name| collection(name) } end |
#command(operation, opts = {}) ⇒ Hash
Execute a command on the database.
155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 |
# File 'lib/mongo/database.rb', line 155 def command(operation, opts = {}) txn_read_pref = opts[:session] && opts[:session].in_transaction? && opts[:session].txn_read_preference Mongo::Lint.validate_underscore_read_preference(txn_read_pref) preference = ServerSelector.get(txn_read_pref || opts[:read] || ServerSelector::PRIMARY) server = preference.select_server(cluster) client.send(:with_session, opts) do |session| Operation::Command.new({ :selector => operation.dup, :db_name => name, :read => preference, :session => session }).execute(server) end end |
#drop(options = {}) ⇒ Result
Drop the database and all its associated information.
183 184 185 186 187 188 189 190 191 192 193 |
# File 'lib/mongo/database.rb', line 183 def drop( = {}) operation = { :dropDatabase => 1 } client.send(:with_session, ) do |session| Operation::DropDatabase.new({ selector: operation, db_name: name, write_concern: write_concern, session: session }).execute(next_primary) end end |
#fs(options = {}) ⇒ Grid::FSBucket
Get the Grid “filesystem” for this database.
234 235 236 |
# File 'lib/mongo/database.rb', line 234 def fs( = {}) Grid::FSBucket.new(self, ) end |
#inspect ⇒ String
Get a pretty printed string inspection for the database.
222 223 224 |
# File 'lib/mongo/database.rb', line 222 def inspect "#<Mongo::Database:0x#{object_id} name=#{name}>" end |
#list_collections ⇒ Array<Hash>
Get info on all the collections in the database.
128 129 130 |
# File 'lib/mongo/database.rb', line 128 def list_collections View.new(self).list_collections end |
#users ⇒ View::User
Get the user view for this database.
246 247 248 |
# File 'lib/mongo/database.rb', line 246 def users Auth::User::View.new(self) end |
#watch(pipeline = [], options = {}) ⇒ ChangeStream
A change stream only allows ‘majority’ read concern.
This helper method is preferable to running a raw aggregation with a $changeStream stage, for the purpose of supporting resumability.
As of version 3.6 of the MongoDB server, a “$changeStream“ pipeline stage is supported in the aggregation framework. As of version 4.0, this stage allows users to request that notifications are sent for all changes that occur in the client’s database.
283 284 285 286 287 288 289 |
# File 'lib/mongo/database.rb', line 283 def watch(pipeline = [], = {}) Mongo::Collection::View::ChangeStream.new( Mongo::Collection::View.new(collection("#{COMMAND}.aggregate")), pipeline, Mongo::Collection::View::ChangeStream::DATABASE, ) end |