Method: Mongo::Database#list_collections
- Defined in:
- lib/mongo/database.rb
#list_collections(options = {}) ⇒ Array<Hash>
Note:
The set of collections returned, and the schema of the information hash per collection, depends on the MongoDB server version that fulfills the request.
Get info on all the non-system collections in the database.
167 168 169 |
# File 'lib/mongo/database.rb', line 167 def list_collections( = {}) View.new(self).list_collections() end |