Class: Mongo::Error::InvalidURI
- Inherits:
-
Error
- Object
- Error
- Mongo::Error::InvalidURI
- Defined in:
- lib/mongo/error/invalid_uri.rb
Overview
Exception that is raised when trying to parse a URI that does not match the specification.
Instance Method Summary collapse
-
#initialize(uri, details, format = nil) ⇒ InvalidURI
constructor
Instantiate the new exception.
Constructor Details
#initialize(uri, details, format = nil) ⇒ InvalidURI
Instantiate the new exception.
The URI is redacted via URI.redact before being interpolated so that any cleartext credentials in the original input do not end up in logs, error reporters, or backtraces.
34 35 36 37 38 39 40 |
# File 'lib/mongo/error/invalid_uri.rb', line 34 def initialize(uri, details, format = nil) = "Bad URI: #{Mongo::URI.redact(uri)}\n" + "#{details}\n" += "MongoDB URI must be in the following format: #{format}\n" if format += "Please see the following URL for more information: #{Mongo::URI::HELP}\n" super() end |