Exception: Mongo::Error::UnsupportedArrayFilters Deprecated
- Inherits:
-
UnsupportedOption
- Object
- StandardError
- Mongo::Error
- UnsupportedOption
- Mongo::Error::UnsupportedArrayFilters
- Defined in:
- lib/mongo/error/unsupported_array_filters.rb
Overview
RUBY-2260 In driver version 3.0, this error class will be replaced with UnsupportedOption. To handle this error, catch Mongo::Error::UnsupportedOption, which will prevent any breaking changes in your application when upgrading to version 3.0 of the driver.
Raised if the array filters option is specified for an operation but the server selected does not support array filters.
Constant Summary collapse
- UNACKNOWLEDGED_WRITES_MESSAGE =
The error message describing that array filters cannot be used when write concern is unacknowledged.
'The array_filters option cannot be specified when using unacknowledged writes. ' + 'Either remove the array_filters option or use acknowledged writes (w >= 1).'
Constants inherited from UnsupportedOption
Mongo::Error::UnsupportedOption::ALLOW_DISK_USE_MESSAGE, Mongo::Error::UnsupportedOption::COMMIT_QUORUM_MESSAGE, Mongo::Error::UnsupportedOption::HINT_MESSAGE, Mongo::Error::UnsupportedOption::UNACKNOWLEDGED_HINT_MESSAGE
Constants inherited from Mongo::Error
BAD_VALUE, CODE, CURSOR_NOT_FOUND, ERRMSG, TRANSIENT_TRANSACTION_ERROR_LABEL, UNKNOWN_ERROR, UNKNOWN_TRANSACTION_COMMIT_RESULT_LABEL, WRITE_CONCERN_ERROR, WRITE_CONCERN_ERRORS, WRITE_ERRORS
Instance Attribute Summary
Attributes included from Notable
#connection_global_id, #generation, #service_id
Method Summary
Methods inherited from UnsupportedOption
allow_disk_use_error, commit_quorum_error, hint_error
Methods inherited from Mongo::Error
#change_stream_resumable?, #initialize, #network_error?, #write_concern_error_label?, #write_concern_error_labels
Methods included from ChangeStreamResumable
Methods included from WriteRetryable
Methods included from Labelable
Methods included from Notable
#add_note, #add_notes, #notes, #to_s
Constructor Details
This class inherits a constructor from Mongo::Error