Class: Mongo::Error::UnsupportedOption
- Inherits:
-
Error
- Object
- Error
- Mongo::Error::UnsupportedOption
- Defined in:
- lib/mongo/error/unsupported_option.rb
Overview
Raised if an unsupported option is specified for an operation.
Direct Known Subclasses
Constant Summary collapse
- ALLOW_DISK_USE_MESSAGE =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
The error message provided when the user passes the allow_disk_use option to a find operation against a server that does not support the allow_disk_use operation and does not provide option validation.
'The MongoDB server handling this request does ' \ 'not support the allow_disk_use option on this command. The ' \ 'allow_disk_use option is supported on find commands on MongoDB ' \ 'server versions 4.4 and later'
- COMMIT_QUORUM_MESSAGE =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
The error message provided when the user passes the commit_quorum option to a createIndexes operation against a server that does not support that option.
'The MongoDB server handling this request does ' \ 'not support the commit_quorum option on this command. The commit_quorum ' \ 'option is supported on createIndexes commands on MongoDB server versions ' \ '4.4 and later'
Class Method Summary collapse
-
.allow_disk_use_error ⇒ Mongo::Error::UnsupportedOption
private
Raise an error about an unsupported allow_disk_use option.
-
.commit_quorum_error ⇒ Mongo::Error::UnsupportedOption
private
Raise an error about an unsupported commit_quorum option.
Class Method Details
.allow_disk_use_error ⇒ Mongo::Error::UnsupportedOption
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Raise an error about an unsupported allow_disk_use option.
47 48 49 |
# File 'lib/mongo/error/unsupported_option.rb', line 47 def self.allow_disk_use_error new(ALLOW_DISK_USE_MESSAGE) end |
.commit_quorum_error ⇒ Mongo::Error::UnsupportedOption
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Raise an error about an unsupported commit_quorum option.
57 58 59 |
# File 'lib/mongo/error/unsupported_option.rb', line 57 def self.commit_quorum_error new(COMMIT_QUORUM_MESSAGE) end |