Class: Mongo::Node
Instance Attribute Summary collapse
-
#address ⇒ Object
Returns the value of attribute address.
-
#client ⇒ Object
Returns the value of attribute client.
-
#config ⇒ Object
Returns the value of attribute config.
-
#host ⇒ Object
Returns the value of attribute host.
-
#last_state ⇒ Object
Returns the value of attribute last_state.
-
#port ⇒ Object
Returns the value of attribute port.
-
#socket ⇒ Object
Returns the value of attribute socket.
Instance Method Summary collapse
- #active? ⇒ Boolean
- #arbiters ⇒ Object
- #close ⇒ Object
-
#connect ⇒ Object
Create a connection to the provided node, and, if successful, return the socket.
- #connected? ⇒ Boolean
- #eql?(other) ⇒ Boolean (also: #==)
- #hash ⇒ Object
- #healthy? ⇒ Boolean
- #host_port ⇒ Object
- #host_string ⇒ Object
-
#initialize(client, host_port) ⇒ Node
constructor
A new instance of Node.
- #inspect ⇒ Object
-
#node_list ⇒ Object
Return a list of replica set nodes from the config.
- #primary? ⇒ Boolean
- #secondary? ⇒ Boolean
-
#set_config ⇒ Object
Get the configuration for the provided node as returned by the ismaster command.
- #tags ⇒ Object
Constructor Details
#initialize(client, host_port) ⇒ Node
Returns a new instance of Node.
6 7 8 9 10 11 12 |
# File 'lib/mongo/util/node.rb', line 6 def initialize(client, host_port) @client = client @host, @port = split_node(host_port) @address = "#{@host}:#{@port}" @config = nil @socket = nil end |
Instance Attribute Details
#address ⇒ Object
Returns the value of attribute address.
4 5 6 |
# File 'lib/mongo/util/node.rb', line 4 def address @address end |
#client ⇒ Object
Returns the value of attribute client.
4 5 6 |
# File 'lib/mongo/util/node.rb', line 4 def client @client end |
#config ⇒ Object
Returns the value of attribute config.
4 5 6 |
# File 'lib/mongo/util/node.rb', line 4 def config @config end |
#host ⇒ Object
Returns the value of attribute host.
4 5 6 |
# File 'lib/mongo/util/node.rb', line 4 def host @host end |
#last_state ⇒ Object
Returns the value of attribute last_state.
4 5 6 |
# File 'lib/mongo/util/node.rb', line 4 def last_state @last_state end |
#port ⇒ Object
Returns the value of attribute port.
4 5 6 |
# File 'lib/mongo/util/node.rb', line 4 def port @port end |
#socket ⇒ Object
Returns the value of attribute socket.
4 5 6 |
# File 'lib/mongo/util/node.rb', line 4 def socket @socket end |
Instance Method Details
#active? ⇒ Boolean
55 56 57 58 59 60 61 62 |
# File 'lib/mongo/util/node.rb', line 55 def active? begin result = @client['admin'].command({:ping => 1}, :socket => @socket) rescue OperationFailure, SocketError, SystemCallError, IOError return nil end result['ok'] == 1 end |
#arbiters ⇒ Object
102 103 104 105 106 107 108 109 110 |
# File 'lib/mongo/util/node.rb', line 102 def arbiters connect unless connected? set_config unless @config return [] unless config['arbiters'] config['arbiters'].map do |arbiter| split_node(arbiter) end end |
#close ⇒ Object
43 44 45 46 47 48 49 |
# File 'lib/mongo/util/node.rb', line 43 def close if @socket && !@socket.closed? @socket.close end @socket = nil @config = nil end |
#connect ⇒ Object
Create a connection to the provided node, and, if successful, return the socket. Otherwise, return nil.
30 31 32 33 34 35 36 37 38 39 40 41 |
# File 'lib/mongo/util/node.rb', line 30 def connect begin socket = @client.socket_class.new(@host, @port, @client.op_timeout, @client.connect_timeout ) rescue OperationTimeout, ConnectionFailure, OperationFailure, SocketError, SystemCallError, IOError => ex @client.log(:debug, "Failed connection to #{host_string} with #{ex.class}, #{ex.}.") socket.close if socket end @socket = socket end |
#connected? ⇒ Boolean
51 52 53 |
# File 'lib/mongo/util/node.rb', line 51 def connected? @socket != nil end |
#eql?(other) ⇒ Boolean Also known as: ==
14 15 16 |
# File 'lib/mongo/util/node.rb', line 14 def eql?(other) other.is_a?(Node) && @address == other.address end |
#hash ⇒ Object
128 129 130 |
# File 'lib/mongo/util/node.rb', line 128 def hash address.hash end |
#healthy? ⇒ Boolean
132 133 134 135 136 137 138 |
# File 'lib/mongo/util/node.rb', line 132 def healthy? if @config.has_key?('secondary') @config['ismaster'] || @config['secondary'] else true end end |
#host_port ⇒ Object
124 125 126 |
# File 'lib/mongo/util/node.rb', line 124 def host_port [@host, @port] end |
#host_string ⇒ Object
19 20 21 |
# File 'lib/mongo/util/node.rb', line 19 def host_string address end |
#inspect ⇒ Object
23 24 25 |
# File 'lib/mongo/util/node.rb', line 23 def inspect "<Mongo::Node:0x#{self.object_id.to_s(16)} @host=#{@host} @port=#{@port}>" end |
#node_list ⇒ Object
Return a list of replica set nodes from the config. Note: this excludes arbiters.
90 91 92 93 94 95 96 97 98 99 100 |
# File 'lib/mongo/util/node.rb', line 90 def node_list connect unless connected? set_config unless @config return [] unless config nodes = [] nodes += config['hosts'] if config['hosts'] nodes += config['passives'] if config['passives'] nodes end |
#primary? ⇒ Boolean
112 113 114 |
# File 'lib/mongo/util/node.rb', line 112 def primary? @config['ismaster'] == true || @config['ismaster'] == 1 end |
#secondary? ⇒ Boolean
116 117 118 |
# File 'lib/mongo/util/node.rb', line 116 def secondary? @config['secondary'] == true || @config['secondary'] == 1 end |
#set_config ⇒ Object
Get the configuration for the provided node as returned by the ismaster command. Additionally, check that the replica set name matches with the name provided.
67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 |
# File 'lib/mongo/util/node.rb', line 67 def set_config begin @config = @client['admin'].command({:ismaster => 1}, :socket => @socket) if @config['msg'] @client.log(:warn, "#{config['msg']}") end check_set_membership(config) check_set_name(config) rescue ConnectionFailure, OperationFailure, OperationTimeout, SocketError, SystemCallError, IOError => ex @client.log(:warn, "Attempted connection to node #{host_string} raised " + "#{ex.class}: #{ex.}") # Socket may already be nil from issuing command close end @config end |
#tags ⇒ Object
120 121 122 |
# File 'lib/mongo/util/node.rb', line 120 def @config['tags'] || {} end |