Module: Mongo::Protocol::Serializers Private
- Included in:
- Message
- Defined in:
- lib/mongo/protocol/serializers.rb,
lib/mongo/protocol/bit_vector.rb
Overview
This module is part of a private API. You should avoid using this module if possible, as it may be removed or be changed in the future.
Container for various serialization strategies
Each strategy must have a serialization method named serialize and a deserialization method named deserialize
Serialize methods must take buffer and value arguments and serialize the value into the buffer
Deserialize methods must take an IO stream argument and deserialize the value from the stream of bytes
Defined Under Namespace
Modules: Byte, Bytes, CString, Document, Header, Int32, Int64, Sections, Zero Classes: BitVector
Constant Summary collapse
- ZERO =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
0- NULL =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
0.chr.freeze
- INT32_PACK =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
'l<'- INT64_PACK =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
'q<'- HEADER_PACK =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
'l<l<l<l<'