Module: Mongo::Protocol::Serializers::Byte Private

Defined in:
lib/mongo/protocol/serializers.rb

Overview

This module is part of a private API. You should avoid using this module if possible, as it may be removed or be changed in the future.

MongoDB wire protocol serialization strategy for a single byte.

Writes and fetches a single byte from the byte buffer.

API:

  • private

Class Method Summary collapse

Class Method Details

.deserialize(buffer, options = {}) ⇒ String

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Deserializes a byte from the byte buffer.

Since:

  • 2.5.0

API:

  • private

Parameters:

  • Buffer containing the value to read.

  • (defaults to: {})

    This method currently accepts no options.

Returns:

  • The byte.



420
421
422
# File 'lib/mongo/protocol/serializers.rb', line 420

def self.deserialize(buffer, options = {})
  buffer.get_byte
end

.serialize(buffer, value, validating_keys = BSON::Config.validating_keys?) ⇒ BSON::ByteBuffer

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Writes a byte into the buffer.

Since:

  • 2.5.0

API:

  • private

Parameters:

  • Buffer to receive the single byte.

  • The byte to write to the buffer.

  • (defaults to: BSON::Config.validating_keys?)

    Whether to validate keys.

Returns:

  • Buffer with serialized value.



408
409
410
# File 'lib/mongo/protocol/serializers.rb', line 408

def self.serialize(buffer, value, validating_keys = BSON::Config.validating_keys?)
  buffer.put_byte(value)
end