Class: Mongo::Server::Connection
- Inherits:
-
ConnectionBase
- Object
- ConnectionCommon
- ConnectionBase
- Mongo::Server::Connection
- Extended by:
- Forwardable
- Includes:
- Id, Monitoring::Publishable, Retryable
- Defined in:
- lib/mongo/server/connection.rb
Overview
This class models the socket connections for servers and their behavior.
Constant Summary collapse
- PING =
Deprecated.
No longer necessary with Server Selection specification.
The ping command.
{ :ping => 1 }.freeze
- PING_OP_MSG =
Deprecated.
No longer necessary with Server Selection specification.
The ping command for an OP_MSG (server versions >= 3.6).
{ :ping => 1, '$db' => Database::ADMIN }.freeze
- PING_MESSAGE =
Deprecated.
No longer necessary with Server Selection specification.
Ping message.
Protocol::Query.new(Database::ADMIN, Database::COMMAND, PING, :limit => -1)
- PING_OP_MSG_MESSAGE =
Deprecated.
No longer necessary with Server Selection specification.
Ping message as an OP_MSG (server versions >= 3.6).
Protocol::Msg.new([], {}, PING_OP_MSG)
- PING_BYTES =
Deprecated.
No longer necessary with Server Selection specification.
The ping message as raw bytes.
PING_MESSAGE.serialize.to_s.freeze
- PING_OP_MSG_BYTES =
Deprecated.
No longer necessary with Server Selection specification.
The ping OP_MSG message as raw bytes (server versions >= 3.6).
PING_OP_MSG_MESSAGE.serialize.to_s.freeze
Constants included from Loggable
Constants inherited from ConnectionBase
Mongo::Server::ConnectionBase::DEFAULT_MAX_BSON_OBJECT_SIZE, Mongo::Server::ConnectionBase::MAX_BSON_COMMAND_OVERHEAD, Mongo::Server::ConnectionBase::REDUCED_MAX_BSON_SIZE
Instance Attribute Summary collapse
-
#global_id ⇒ Integer
readonly
across all connections.
-
#id ⇒ Integer
readonly
across connections to the same server object.
-
#last_checkin ⇒ Time
readonly
The last time the connection was checked back into a pool.
Attributes included from Monitoring::Publishable
Attributes inherited from ConnectionBase
#description, #options, #server
Attributes inherited from ConnectionCommon
Instance Method Summary collapse
-
#closed? ⇒ true | false
Whether the connection was closed.
-
#connect! ⇒ true
Establishes a network connection to the target address.
-
#connection_pool ⇒ Object
private
The connection pool from which this connection was created.
-
#disconnect!(options = nil) ⇒ true
Disconnect the connection.
- #error? ⇒ Boolean private
-
#initialize(server, options = {}) ⇒ Connection
constructor
private
Creates a new connection object to the specified target address with the specified options.
-
#pin ⇒ Object
private
Mark the connection as pinned.
-
#ping ⇒ true, false
deprecated
Deprecated.
No longer necessary with Server Selection specification.
-
#pinned? ⇒ Boolean
private
Whether the connection is used by a transaction or cursor operations.
-
#record_checkin! ⇒ self
Record the last checkin time.
-
#socket_timeout ⇒ Float
(also: #timeout)
Get the timeout to execute an operation on a socket.
-
#unpin ⇒ Object
private
Mark the connection as not pinned.
Methods included from Id
Methods included from Retryable
#legacy_write_with_retry, #nro_write_with_retry, #read_with_one_retry, #read_with_retry, #read_with_retry_cursor, #write_with_retry
Methods included from Monitoring::Publishable
#publish_cmap_event, #publish_event, #publish_sdam_event
Methods included from Loggable
#log_debug, #log_error, #log_fatal, #log_info, #log_warn, #logger
Methods inherited from ConnectionBase
#app_metadata, #dispatch, #generation, #service_id
Methods inherited from ConnectionCommon
#connected?, #handshake_command, #handshake_document
Constructor Details
#initialize(server, options = {}) ⇒ Connection
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Connection must never be directly instantiated outside of a Server.
Creates a new connection object to the specified target address with the specified options.
The constructor does not perform any I/O (and thus does not create sockets, handshakes nor authenticates); call connect! method on the connection object to create the network connection.
103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 |
# File 'lib/mongo/server/connection.rb', line 103 def initialize(server, = {}) if server.load_balancer? && [:generation] raise ArgumentError, "Generation cannot be set when server is a load balancer" end @id = server.next_connection_id @global_id = self.class.next_id @monitoring = server.monitoring @options = .freeze @server = server @socket = nil @last_checkin = nil @auth_mechanism = nil @pid = Process.pid @pinned = false publish_cmap_event( Monitoring::Event::Cmap::ConnectionCreated.new(address, id) ) end |
Instance Attribute Details
#global_id ⇒ Integer (readonly)
across all connections.
137 138 139 |
# File 'lib/mongo/server/connection.rb', line 137 def global_id @global_id end |
#id ⇒ Integer (readonly)
across connections to the same server object.
133 134 135 |
# File 'lib/mongo/server/connection.rb', line 133 def id @id end |
#last_checkin ⇒ Time (readonly)
Returns The last time the connection was checked back into a pool.
127 128 129 |
# File 'lib/mongo/server/connection.rb', line 127 def last_checkin @last_checkin end |
Instance Method Details
#closed? ⇒ true | false
Whether the connection was closed.
Closed connections should no longer be used. Instead obtain a new connection from the connection pool.
155 156 157 |
# File 'lib/mongo/server/connection.rb', line 155 def closed? !!@closed end |
#connect! ⇒ true
This method mutates the connection object by setting a socket if one previously did not exist.
Establishes a network connection to the target address.
If the connection is already established, this method does nothing.
203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 |
# File 'lib/mongo/server/connection.rb', line 203 def connect! if error? raise Error::ConnectionPerished, "Connection #{generation}:#{id} for #{address.seed} is perished. Reconnecting closed or errored connections is no longer supported" end if closed? raise Error::ConnectionPerished, "Connection #{generation}:#{id} for #{address.seed} is closed. Reconnecting closed or errored connections is no longer supported" end unless @socket # When @socket is assigned, the socket should have handshaken and # authenticated and be usable. @socket, @description, @compressor = do_connect if server.load_balancer? if Lint.enabled? unless service_id raise Error::InternalDriverError, "The connection is to a load balancer and it must have service_id set here, but does not" end end @generation = connection_pool.generation_manager.generation(service_id: service_id) end publish_cmap_event( Monitoring::Event::Cmap::ConnectionReady.new(address, id) ) @close_event_published = false end true end |
#connection_pool ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
The connection pool from which this connection was created. May be nil.
143 144 145 |
# File 'lib/mongo/server/connection.rb', line 143 def connection_pool [:connection_pool] end |
#disconnect!(options = nil) ⇒ true
Once a connection is disconnected, it should no longer be used. A new connection should be obtained from the connection pool which will either return a ready connection or create a new connection. If linting is enabled, reusing a disconnected connection will raise Error::LintError. If linting is not enabled, a warning will be logged.
This method mutates the connection object by setting the socket to nil if the closing succeeded.
Disconnect the connection.
275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 |
# File 'lib/mongo/server/connection.rb', line 275 def disconnect!( = nil) # Note: @closed may be true here but we also may have a socket. # Check the socket and not @closed flag. @auth_mechanism = nil @last_checkin = nil if socket socket.close rescue nil @socket = nil end @closed = true # To satisfy CMAP spec tests, publish close events even if the # socket was never connected (and thus the ready event was never # published). But track whether we published close event and do not # publish it multiple times, unless the socket was reconnected - # in that case publish the close event once per socket close. unless @close_event_published reason = && [:reason] publish_cmap_event( Monitoring::Event::Cmap::ConnectionClosed.new( address, id, reason, ), ) @close_event_published = true end true end |
#error? ⇒ Boolean
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
160 161 162 |
# File 'lib/mongo/server/connection.rb', line 160 def error? !!@error end |
#pin ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Mark the connection as pinned.
179 180 181 |
# File 'lib/mongo/server/connection.rb', line 179 def pin @pinned = true end |
#ping ⇒ true, false
No longer necessary with Server Selection specification.
This uses a pre-serialized ping message for optimization.
Ping the connection to see if the server is responding to commands. This is non-blocking on the server side.
319 320 321 322 323 324 325 326 327 328 |
# File 'lib/mongo/server/connection.rb', line 319 def ping bytes = features.op_msg_enabled? ? PING_OP_MSG_BYTES : PING_BYTES ensure_connected do |socket| reply = add_server_diagnostics do socket.write(bytes) Protocol::Message.deserialize(socket, ) end reply.documents[0][Operation::Result::OK] == 1 end end |
#pinned? ⇒ Boolean
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Whether the connection is used by a transaction or cursor operations.
Pinned connections should not be disconnected and removed from a connection pool if they are idle or stale.
# @return [ true | false ] Whether connection is pinned.
172 173 174 |
# File 'lib/mongo/server/connection.rb', line 172 def pinned? @pinned end |
#record_checkin! ⇒ self
Record the last checkin time.
349 350 351 352 |
# File 'lib/mongo/server/connection.rb', line 349 def record_checkin! @last_checkin = Time.now self end |
#socket_timeout ⇒ Float Also known as: timeout
Get the timeout to execute an operation on a socket.
335 336 337 |
# File 'lib/mongo/server/connection.rb', line 335 def socket_timeout @timeout ||= [:socket_timeout] end |
#unpin ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Mark the connection as not pinned.
186 187 188 |
# File 'lib/mongo/server/connection.rb', line 186 def unpin @pinned = false end |