Module: Mongo::Support
- Extended by:
- Support
- Includes:
- Conversions
- Included in:
- Support
- Defined in:
- lib/mongo/util/support.rb
Constant Summary collapse
- READ_PREFERENCES =
[ :primary, :primary_preferred, :secondary, :secondary_preferred, :nearest ]
- SECONDARY_OK_COMMANDS =
Commands that may be sent to replica-set secondaries, depending on read preference and tags. All other commands are always run on the primary.
[ 'group', 'aggregate', 'collstats', 'dbstats', 'count', 'distinct', 'geonear', 'geosearch', 'geowalk' ]
Constants included from Conversions
Conversions::ASCENDING_CONVERSION, Conversions::DESCENDING_CONVERSION
Instance Method Summary collapse
-
#auth_key(username, password, nonce) ⇒ String
Generate an MD5 for authentication.
- #format_order_clause(order) ⇒ Object
-
#hash_password(username, plaintext) ⇒ String
Return a hashed password for auth.
-
#ok?(doc) ⇒ Boolean
Determine if a database command has succeeded by checking the document response.
- #secondary_ok?(selector) ⇒ Boolean
- #validate_db_name(db_name) ⇒ Object
- #validate_read_preference(value) ⇒ Object
Methods included from Conversions
#array_as_sort_parameters, #hash_as_sort_parameters, #sort_value, #string_as_sort_parameters
Instance Method Details
#auth_key(username, password, nonce) ⇒ String
Generate an MD5 for authentication.
56 57 58 |
# File 'lib/mongo/util/support.rb', line 56 def auth_key(username, password, nonce) Digest::MD5.hexdigest("#{nonce}#{username}#{hash_password(username, password)}") end |
#format_order_clause(order) ⇒ Object
106 107 108 109 110 111 112 113 114 115 |
# File 'lib/mongo/util/support.rb', line 106 def format_order_clause(order) case order when Hash, BSON::OrderedHash then hash_as_sort_parameters(order) when String, Symbol then string_as_sort_parameters(order) when Array then array_as_sort_parameters(order) else raise InvalidSortValueError, "Illegal sort clause, '#{order.class.name}'; must be of the form " + "[['field1', '(ascending|descending)'], ['field2', '(ascending|descending)']]" end end |
#hash_password(username, plaintext) ⇒ String
Return a hashed password for auth.
66 67 68 |
# File 'lib/mongo/util/support.rb', line 66 def hash_password(username, plaintext) Digest::MD5.hexdigest("#{username}:mongo:#{plaintext}") end |
#ok?(doc) ⇒ Boolean
Determine if a database command has succeeded by checking the document response.
123 124 125 |
# File 'lib/mongo/util/support.rb', line 123 def ok?(doc) doc['ok'] == 1.0 || doc['ok'] == true end |
#secondary_ok?(selector) ⇒ Boolean
84 85 86 87 88 89 90 91 92 93 94 95 |
# File 'lib/mongo/util/support.rb', line 84 def secondary_ok?(selector) command = selector.keys.first.to_s.downcase if command == 'mapreduce' out = selector.select { |k, v| k.to_s.downcase == 'out' }.first.last # mongo looks at the first key in the out object, and doesn't # look at the value out.is_a?(Hash) && out.keys.first.to_s.downcase == 'inline' ? true : false else SECONDARY_OK_COMMANDS.member?(command) end end |
#validate_db_name(db_name) ⇒ Object
70 71 72 73 74 75 76 77 78 79 80 81 82 |
# File 'lib/mongo/util/support.rb', line 70 def validate_db_name(db_name) unless [String, Symbol].include?(db_name.class) raise TypeError, "db_name must be a string or symbol" end [" ", ".", "$", "/", "\\"].each do |invalid_char| if db_name.include? invalid_char raise Mongo::InvalidNSName, "database names cannot contain the character '#{invalid_char}'" end end raise Mongo::InvalidNSName, "database name cannot be the empty string" if db_name.empty? db_name end |
#validate_read_preference(value) ⇒ Object
97 98 99 100 101 102 103 104 |
# File 'lib/mongo/util/support.rb', line 97 def validate_read_preference(value) if READ_PREFERENCES.include?(value) return true else raise MongoArgumentError, "#{value} is not a valid read preference. " + "Please specify one of the following read preferences as a symbol: #{READ_PREFERENCES}" end end |